#!BPY

"""
Name: 'Bounding Convex Hull'
Blender: 242
Group: 'Object'
Tooltip: 'Make a bounding convex hull'
"""


__author__ = "Adrian Boeing"
__url__ = ("http://www.blender.org", "http://www.transmin.com.au", "http://www.adrianboeing.com")
__version__ = "1.0 26/10/2010"

__bpydoc__ = """\
This script creates a bounding convex hull of the selected object.

Usage:

Select one object and run this script. A new object is created that represents a convex-hull of the selected object. The monikor ".hull" is appended to its name.

Use this script if you want a convex hull representation of your object, for use with physics engines, collision detection or other computer graphics and robotics applications.

This script will only work with Mesh object types. Internally it generates an obj file and re-imports an obj file after conversion. You must have the 'convex' C++ program in Blenders path.
"""

# $Id: object_boundinghull.py$
#
# --------------------------------------------------------------------------
# ***** BEGIN GPL LICENSE BLOCK *****
#
# Copyright (C) 2010: Adrian Boeing. http://www.adrianboeing.com/
#
# Thanks to Stan Melax for the original convex hull generation code,
# and John Ratclif for the C++ version. The C++ code was modified by
# Adrian Boeing for the PAL project. 
# http://pal.sf.net/
# The Blender conversion was writen by Adrian Boeing for Transmin Pty Ltd 
# http://www.transmin.com.au/
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import subprocess
import bpy
import Blender
from Blender import *

def write_obj(filepath,mesh):
	out = file(filepath, 'w')
	for vert in mesh.verts:
		out.write( 'v %f %f %f\n' % (vert.co.x, vert.co.y, vert.co.z) )
	
	for face in mesh.faces:
		out.write('f')
		
		for vert in face.v:
			out.write( ' %i' % (vert.index + 1) )
		out.write('\n')
	out.close()
	
def import_obj(path, name):
        mesh = Blender.NMesh.New( name ) # create a new mesh
        # parse the file
        file = open(path, 'r')
        for line in file:
                words = line.split()
                if len(words) == 0 or words[0].startswith('#'):
                        pass
                elif words[0] == 'v':
                        x, y, z = float(words[1]), float(words[2]), float(words[3])
                        mesh.verts.append(Blender.NMesh.Vert(x, y, z))
                elif words[0] == 'f':
                        faceVertList = []
                        for faceIdx in words[1:]:
                                faceVert = mesh.verts[int(faceIdx)-1]
                                faceVertList.append(faceVert)
                        newFace = Blender.NMesh.Face(faceVertList)
                        mesh.addFace(newFace)
        
        # link the mesh to a new object
        ob = Blender.Object.New('Mesh', name) # Mesh must be spelled just this--it is a specific type
        ob.link(mesh) # tell the object to use the mesh we just made
        scn = Blender.Scene.GetCurrent()
        for o in scn.getChildren():
                o.sel = 0
        
        scn.link(ob) # link the object to the current scene
        ob.sel= 1
        ob.Layers = scn.Layers
        return ob
    
    
def main():
    scn = bpy.data.scenes.active
    ob = scn.objects.active
    if not ob:
        Blender.Draw.PupMenu('No object selected.')
        return
    if ob.type != 'Mesh':
			Draw.PupMenu("Error%t|Selection must be made up of mesh objects only")
			return
            
    result = Draw.PupMenu("Create bounding hull?%t|OK")
    if result == -1:
        return
        
    mesh = ob.getData(mesh=1)
    write_obj("tmp.obj",mesh)
    
    subprocess.call("./convex tmp.obj", shell = True)
    
    newob = import_obj("out.obj",ob.name + ".hull")
    
    #print dir(ob)
    
    newob.LocX = ob.LocX
    newob.LocY = ob.LocY
    newob.LocZ = ob.LocZ
    newob.RotX = ob.RotX
    newob.RotY = ob.RotY
    newob.RotZ = ob.RotZ
    newob.SizeX = ob.SizeX
    newob.SizeY = ob.SizeY
    newob.SizeZ = ob.SizeZ
    
if __name__=='__main__':
	main()
